package rpc

import (
	"context"
	"io"

	"google.golang.org/grpc/metadata"

	"git.loafle.net/commons_go/rpc/gateway"
	"git.loafle.net/commons_go/rpc/protocol/json"
	oogw "git.loafle.net/overflow/overflow_gateway_websocket"
	"git.loafle.net/overflow/overflow_gateway_websocket/external/grpc"
)

func newRPCGatewayServerHandler() RPCGatewayServerHandler {
	sh := &RPCGatewayServerHandlers{}
	sh.RegisterCodec(json.NewServerCodec(), json.Name)
	return sh
}

type RPCGatewayServerHandler interface {
	gateway.ServerHandler
}

type RPCGatewayServerHandlers struct {
	gateway.ServerHandlers
}

func (sh *RPCGatewayServerHandlers) OnPreRead(r io.Reader) {
	// no op
}

func (sh *RPCGatewayServerHandlers) OnPostRead(r io.Reader) {
	// no op
}

func (sh *RPCGatewayServerHandlers) OnPreWriteResult(w io.Writer, result interface{}) {
	// no op
}

func (sh *RPCGatewayServerHandlers) OnPostWriteResult(w io.Writer, result interface{}) {
	// no op
}

func (sh *RPCGatewayServerHandlers) OnPreWriteError(w io.Writer, err error) {
	// no op
}

func (sh *RPCGatewayServerHandlers) OnPostWriteError(w io.Writer, err error) {
	// no op
}

func (sh *RPCGatewayServerHandlers) Invoke(ctx context.Context, method string, params []string) (result interface{}, err error) {
	soc := ctx.Value(oogw.ServletSocketKey).(Socket)

	md := metadata.Pairs(oogw.GRPCUserIDKey, soc.ID())
	grpcCTX := metadata.NewOutgoingContext(context.Background(), md)

	r, err := grpc.Exec(grpcCTX, method, params)
	if err != nil {
		return nil, err
	}

	return r, nil

}

func (sh *RPCGatewayServerHandlers) Validate() {
	sh.ServerHandlers.Validate()
}
